﻿using System.Text.RegularExpressions;
using System.Xml;
using Tridion.ContentManager;
using Tridion.ContentManager.CommunicationManagement;
using Tridion.ContentManager.Templating;
using Tridion.ContentManager.Templating.Assembly;

namespace Trivident.DD4T.Templates
{
    [TcmTemplateTitle("Distributed Sitemap")]
    public class Sitemap : ITemplate
    {
        private Page page { get; set; }
        public void Transform(Engine engine, Package package)
        {

            Item pageItem = package.GetByType(ContentType.Page);
            if (pageItem == null)
            {
                return;

            }
            page = engine.GetObject(pageItem.GetAsSource().GetValue("ID")) as Page;
            Publication publication = page.ContextRepository as Publication;

            XmlDocument sitemap = null; 
            sitemap = new XmlDocument();
            sitemap.LoadXml("<siteMap xmlns=\"http://schemas.microsoft.com/AspNet/SiteMap-File-1.0\" />");

            string baseUrl = null;
            StructureGroup startSG = null;
            if (page.OrganizationalItem.Id == publication.RootStructureGroup.Id || page.OrganizationalItem.Title.ToLower() == "system")
            {
                baseUrl = publication.PublicationUrl;
                startSG = publication.RootStructureGroup; 
            }
            else
            {
                baseUrl = ((StructureGroup)page.OrganizationalItem).PublishLocationUrl;
                startSG = page.OrganizationalItem as StructureGroup;
            }

            Regex trimSlash = new Regex("/$");
            baseUrl = trimSlash.Replace(baseUrl, "") + "/";

            XmlElement navigation = CreateElement(sitemap.DocumentElement, "navigation", baseUrl, startSG.Title, startSG.Title);
            TraverseSG(navigation, startSG, page.Session, baseUrl);

            package.PushItem("Output", package.CreateStringItem(ContentType.Xml, sitemap.OuterXml));
        }

        private void TraverseSG(XmlElement context, StructureGroup sg, Session session, string baseUrl)
        {
            // first look for possible (sub)navigation pages
            // if there is one, this entire SG will be part of the subnavigation, so none of the other pages must be added to the navigation from here!
            foreach (Page childPage in sg.GetItems(new Tridion.ContentManager.ContentManagement.OrganizationalItemItemsFilter(session) { ItemTypes = new[] { ItemType.Page } }))
            {
                if (childPage.Id == page.Id) // always skip 'ourselves'
                    continue;
                if (childPage.FileName.ToLower() == "navigation")
                {
                    context.SetAttribute("subnavigation", baseUrl + childPage.FileName + "." + childPage.PageTemplate.FileExtension);
                    return;
                }
            }

            // the SG does not contain subnavigation, now proceed with normal operation
            foreach (Page childPage in sg.GetItems(new Tridion.ContentManager.ContentManagement.OrganizationalItemItemsFilter(session) { ItemTypes = new[] { ItemType.Page } }))
            {
                if (childPage.Id == page.Id) // always skip 'ourselves'
                    continue;
                if (childPage.FileName == "index")
                { 
                    context.SetAttribute("id", childPage.Id);
                }
                else
                {
                    XmlElement childElmt = CreateElement(context, childPage.Id, baseUrl + childPage.FileName + "." + childPage.PageTemplate.FileExtension, childPage.Title, childPage.Title);
                }
            }
            foreach (StructureGroup childSG in sg.GetItems(new Tridion.ContentManager.ContentManagement.OrganizationalItemItemsFilter(session) { ItemTypes = new[] { ItemType.StructureGroup } }))
            {
                XmlElement childElmt = CreateElement(context, childSG.Id, baseUrl + childSG.Directory + "/", childSG.Title, childSG.Title);
                TraverseSG(childElmt, childSG, session, baseUrl + childSG.Directory + "/");
            }
        }

        private static XmlElement CreateElement(XmlElement context, string id, string url, string title, string description)
        {
            XmlElement result = context.OwnerDocument.CreateElement("siteMapNode");

            result.SetAttribute("id", id);
            result.SetAttribute("url", url);
            result.SetAttribute("title", title);
            result.SetAttribute("description", description);
            context.AppendChild(result);

            return result;
        }

    }
}
