﻿namespace Mvc4Demo.Sitemap
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Collections.Specialized;
    using System.Web;
    using System.Xml.Linq;
    using DD4T.ContentModel.Contracts.Caching;
    using DD4T.ContentModel.Factories;
    using DD4T.Factories;
    using DD4T.Factories.Caching;
    using DD4T.Mvc.Providers;
    using DD4T.Utils;

    public class DistributedSiteMapProvider : StaticSiteMapProvider
    {

        private IPageFactory _pageFactory = null;
        private ICacheAgent _cacheAgent = null;
        private object lock1 = new object();



        public const string CacheRegion = "System";
        public const string CacheKey = "SiteMap";
        public const string CacheNullValue = "NULL";
        public const string DefaultSiteMapPath = "/system/sitemap.xml";

        public virtual IPageFactory PageFactory
        {
            get
            {
                if (_pageFactory == null)
                    _pageFactory = new PageFactory();
                return _pageFactory;
            }
            set
            {
                _pageFactory = value;
            }
        }

        /// <summary>
        /// Get or set the CacheAgent
        /// </summary>  
        public virtual ICacheAgent CacheAgent
        {
            get
            {
                if (_cacheAgent == null)
                {
                    _cacheAgent = new DefaultCacheAgent();
                }
                return _cacheAgent;
            }
            set
            {
                _cacheAgent = value;
            }
        }

        public Dictionary<string, SiteMapNode> NodeDictionary { get; set; }

        public virtual string SiteMapPath
        {
            get
            {
                string path = ConfigurationHelper.SiteMapPath;
                if (string.IsNullOrEmpty(path))
                {
                    return DefaultSiteMapPath;
                }
                return path;
            }
        }

        private void ReadSitemapFromXml(string sitemapUrl, SiteMapNode rootNode)
        {
            NodeDictionary = new Dictionary<string, SiteMapNode>();

            string sitemap;
            if (!PageFactory.TryFindPageContent(sitemapUrl, out sitemap))
            {
                return;
            }

            XDocument xDoc = XDocument.Parse(sitemap);
            AddChildren(rootNode, xDoc.Root.Elements(), 1);

        }

        private void AddChildren(SiteMapNode rootNode, IEnumerable<XElement> siteMapNodes, int currentLevel)
        {
            foreach (var element in siteMapNodes)
            {
                DistributedSiteMapNode childNode = new DistributedSiteMapNode(this, element) { Level = currentLevel };
                NodeDictionary.Add(childNode.Key, childNode);

                if (childNode.SubnavigationUrl != null)
                {
                    // if there is a subnavigation, load a separate sitemap instead
                    ReadSitemapFromXml(childNode.SubnavigationUrl, rootNode);
                    continue;
                }
                AddNode(childNode, rootNode);

                // repeat procedure for all children in this node
                AddChildren(childNode, element.Elements(), currentLevel + 1);
            }
        }

        public virtual bool IsInitialized
        {
            get;
            private set;
        }

        public override void Initialize(string name, NameValueCollection attributes)
        {
            if (!IsInitialized)
            {
                base.Initialize(name, attributes);
                IsInitialized = true;
            }
        }

        public override SiteMapNode BuildSiteMap()
        {
            object result = CacheAgent.Load(CacheKey);
            if (result != null)
            {
                if (result is string && ((string)result).Equals(CacheNullValue))
                    return null;
                return result as SiteMapNode;
            }
            SiteMapNode rootNode;
            lock (lock1)
            {
                base.Clear();
                rootNode = new TridionSiteMapNode(this, String.Empty, "root_" + PageFactory.PageProvider.PublicationId, String.Empty, String.Empty, String.Empty, new ArrayList(), new NameValueCollection(), new NameValueCollection(), String.Empty);
                AddNode(rootNode);

                ReadSitemapFromXml(SiteMapPath, rootNode);
                CacheAgent.Store(CacheKey, CacheRegion, rootNode);
            }

            return rootNode;
        }

        protected override SiteMapNode GetRootNodeCore()
        {
            return BuildSiteMap();
        }

        public override SiteMapNode RootNode
        {
            get
            {
                return BuildSiteMap();
            }
        }

        protected override void Clear()
        {
            lock (this)
            {
                // CacheAgent.Remove("rootNode"); // currently, CacheAgents do not support 'Remove'
                base.Clear();
            }
        }
    }
}
 
